#include "MC56F8367.h"
#include "AD1.h"
#include "test.h"

static void HWEnDi(void);
void AD1_Init(void);
byte AD1_Enable(void);
byte AD1_EnableEvent(void);
byte AD1_Start(void);
byte AD1_Measure(bool WaitForResult);
byte AD1_GetValue(void* Values);
asm void Cpu_Delay100US(word);

#define IDLE            0              /* IDLE state           */
#define MEASURE         1              /* MESURE state         */
#define CONTINUOUS      2              /* CONTINUOUS state     */
#define SINGLE          3              /* SINGLE state         */

bool AD1_EnUser = TRUE;                       /* Enable/Disable device */

static bool EnEvent;                   /* Enable/Disable events */
static bool OutFlg;                    /* Measurement finish flag */
/*lint -esym(765,AD1_ModeFlg) Disable MISRA rule (23) checking for symbols (AD1_ModeFlg). */
volatile byte AD1_ModeFlg;             /* Current state of device */
volatile word OnEnd;

void main(void)
{
	test_sRec testRec;
	word wordTmp;
	word wordVal, wordchVal, word16Val, word16chVal;
	byte byteVal, bytechVal;
	word wordVal1, wordchVal1, word16Val1, word16chVal1;
	byte byteVal1, bytechVal1;

	AD1_Init();
	AD1_Enable();
	AD1_EnableEvent();
	AD1_Start();
	AD1_Measure(TRUE);
	//AD1_GetValue(&wordVal);
}


void AD1_Init(void)
{
  AD1_EnUser = TRUE;                   /* Enable device */
  EnEvent = TRUE;                      /* Enable events */
  OutFlg = FALSE;                      /* No measured value */
  AD1_ModeFlg = IDLE;                  /* Device isn't running */
//  setReg(ADCA_ADCR1,20226);
  setReg(ADCA_ADCR1,16386);            /* Set control register 1 */
  setReg(ADCB_ADCR1,16386);            /* Set control register 1 */
  
  setReg(ADCA_ADOFS0,0);               /* Set offset reg. 0 */
  setReg(ADCB_ADOFS0,0);               /* Set offset reg. 0 */

  setReg(ADCA_ADHLMT0,32760);          /* Set high limit reg. 0 */
  setReg(ADCB_ADHLMT0,32760);          /* Set high limit reg. 0 */
  
  setReg(ADCA_ADLLMT0,0);              /* Set low limit reg. 0 */
  setReg(ADCB_ADLLMT0,0);              /* Set low limit reg. 0 */
  
  setReg(ADCA_ADZCSTAT,65535);         /* Clear zero crossing status flags */
  setReg(ADCB_ADZCSTAT,65535);         /* Clear zero crossing status flags */
  
  setReg(ADCA_ADLSTAT,65535);          /* Clear high and low limit status */
  setReg(ADCB_ADLSTAT,65535);          /* Clear high and low limit status */
 
  //setReg(ADCA_ADSTAT,2048);
  setReg(ADCA_ADSTAT,0);            /* Clear EOSI flag */
  setReg(ADCB_ADSTAT,0);            /* Clear EOSI flag */
  
  setReg(ADCA_ADSDIS,0);             /* Enable all samples */
  setReg(ADCB_ADSDIS,0);             /* Enable all samples */
  
  setReg(ADCA_ADLST1,12816);               /* in sample 0 select channel 0 , in sample 1 select channel 1..... */
  setReg(ADCB_ADLST1,12816);               /* in sample 0 select channel 0 , in sample 1 select channel 1..... */
  
  setReg(ADCA_ADLST1,30292);               /* in sample 4 select channel 4 , in sample 5 select channel 5......*/
  setReg(ADCB_ADLST1,30292);               /* in sample 4 select channel 4 , in sample 5 select channel 5......*/
  
  setReg(ADCA_ADZCC,0);                /* Set zero crossing control reg. */
  setReg(ADCB_ADZCC,0);                /* Set zero crossing control reg. */
  
  setReg(ADCA_ADCR2,30);               /* Set prescaler */
  setReg(ADCB_ADCR2,30);               /* Set prescaler */
  
  HWEnDi();                            /* Enable/disable device according to the status flags */
}

byte AD1_Enable(void)
{
  if (AD1_EnUser) {                    /* Is the device enabled by user? */
    return ERR_OK;                     /* If yes then set the flag "device enabled" */
  }
  AD1_EnUser = TRUE;                   /* Set the flag "device enabled" */
  HWEnDi();                            /* Enable the device */
  return ERR_OK;                       /* OK */
}

byte AD1_EnableEvent(void)
{
  EnEvent = TRUE;                      /* Set the flag "events enabled" */
  return ERR_OK;                       /* OK */
}

byte AD1_Start(void)
{
  if (!AD1_EnUser) {                   /* Is the device disabled by user? */
    return ERR_DISABLED;               /* If yes then error */
  }
  if (AD1_ModeFlg != IDLE) {           /* Is the device in running mode? */
    return ERR_BUSY;                   /* If yes then error */
  }
  AD1_ModeFlg = CONTINUOUS;            /* Set state of device to the continuous mode */
  HWEnDi();                            /* Enable the device */
  return ERR_OK;                       /* OK */
}


byte AD1_Measure(bool WaitForResult)
{
  if (!AD1_EnUser) {                   /* Is the device disabled by user? */
    return ERR_DISABLED;               /* If yes then error */
  }
  if (AD1_ModeFlg != IDLE) {           /* Is the device in running mode? */
    return ERR_BUSY;                   /* If yes then error */
  }
  AD1_ModeFlg = MEASURE;               /* Set state of device to the measure mode */
  HWEnDi();                            /* Enable the device */
  if (WaitForResult) {                 /* Is WaitForResult TRUE? */
    while (AD1_ModeFlg == MEASURE) {}  /* If yes then wait for end of measurement */
  }
  return ERR_OK;                       /* OK */
}


static void HWEnDi(void)
{
  volatile word i;

  if (AD1_EnUser) {                    /* Enable device? */
    if ((!getRegBits(ADCA_ADPOWER , 0x00d1)) && (!getRegBits(ADCB_ADPOWER , 0x00d1))) { /* Is device enabled? */
      /* ADCA_ADPOWER: ??=0,??=0,??=0,PSTS2=0,PSTS1=0,PSTS0=0,PUDELAY=13,PSM=1,PD2=0,PD1=0,PD0=0 */
      setReg(ADCA_ADPOWER,216);        /* Enable device */
      setReg(ADCB_ADPOWER,216);
      
      for (i=0;i<100;i++) {}           /* Recovery time of voltage reference */
    }
    if (AD1_ModeFlg) {                 /* Launch measurement? */
      OutFlg = FALSE;                  /* Measured values are available */
      clrRegBits(ADCA_ADCR1, 0x4000);      /* Normal operation mode */
      setRegBits(ADCA_ADCR1, 0x2000);     /* Launching of conversion */
      
      clrRegBits(ADCB_ADCR1, 0x4000);      /* Normal operation mode */
      setRegBits(ADCB_ADCR1, 0x2000);     /* Launching of conversion */
    }
    else {
      setRegBits(ADCA_ADCR1, 0x2000);      /* Stop command issued */
      setRegBits(ADCB_ADCR1, 0x2000);      /* Stop command issued */
    }
  }
  else {
    setRegBits(ADCA_ADCR1, 0x2000);        /* Stop command issued */
    
    setRegBits(ADCB_ADCR1, 0x2000);        /* Stop command issued */
    /* ADCA_ADPOWER: ??=0,??=0,??=0,PSTS2=0,PSTS1=0,PSTS0=0,PUDELAY=13,PSM=0,PD2=1,PD1=1,PD0=1 */
    setReg(ADCA_ADPOWER,215);          /* Disable device */
    setReg(ADCB_ADPOWER,215);          /* Disable device */
  }
}
